/* DRA818V_VHF_DL3ED
(c) DL3ED Ulrich Rumeswinkel
Nokia 5110 LCD Displays
Software mit dem Mini Pro Board 3,3 Volt 8MHz
==============================================================================
von DL3ED (c) 19.3.2015, letzte Änderung 4.4.2015
Bei dem ersten Start der Software muss die Setuptaste gedrückt und dann
einschalten werden.
An den Zeilen mit !!!!!!!!!!!!!!!!! müssen die richtigen Pins angegeben werden.
==============================================================================
Mit der Setup Taste kann folgende Einstellung mit der up, down Taste durchgefürt
werden.
1.Lautstärke 1-8
2.CTCSS 00,0Hz - 250,3Hz
3.Squelch 1-8
4.Tx Power Hi Low, 300mW oder 1Watt
5.Bandbreite schmal breit, (12,5KHz oder 25KHz)
6.Contrasteinstellung des Displays
7.Betriebsspannungsanzeige
Die Werte werden im EEProm gespeichert.

Nur mit der up,down Taste wird die Frequenz von 134MHz bis 144MHz im 10 KHz Raster,
von 144MHz bis 146MHz im 12,5 KHz Raster und von 146MHz bis 174MHz Raster eingestellt.
Durch längeres Drücken der Taster up, down wird die Geschwindigkeit der Frequenz-
verstellung beschleunigt.

Als Ladegerät verwende ich das Graupner/SJ 33032.4, 4,2Volt 500mA

Display Anzeige
145,275          |  Frequenz
Power Low   B.B  |  Leistung Low oder Hi, Bandbreite (schmal oder breit)
SH.0  Sq.1  L.2  |  Shift (0 oder -), Squelch (0-8), Lautstärke (0-8)
CTCSS 00,0  Hz   |  CTCSS Ton 0-250,3Hz 

==============================================================================

!!!!!!!!!!!!        Geändert auf neues Display, je nach Display müssen die Pinne geändert werden.
!!!!!!!!!!!!        Für die Änderung sind die Stellen mit dem !!!!!!! gekennzeichnet.

Pinbelegung der Displays
org   Display   Arduino Micro   neues Display
1 =   VCC =     3,3Volt       =     6      
2 =   GND =     GND           =     8
3 =   CE  =     Arduino 7     =     2
4 =   RST =     Arduino 6     =     1
5 =   DC  =     Arduino 5     =     3
6 =   DIN  =    Arduino 9     =     4
7 =   CLK =     Arduino 8     =     5
8 =   LED =     GND           =     7

Output Power Pin 7  Arduino
Taster Setup Pin 10 Arduino
Taster Down  Pin 8  Arduino
Taster UP    Pin 9  Arduino
Output Led   Pin 13 Arduino

****************************************************************************************************/

#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>
#include <EEPROM.h>

Adafruit_PCD8544 display = Adafruit_PCD8544(6, 5, 4,  3, 2);            //CLK, DIN, DC, CE, RST !!!!!!!!!!!!!!!!!!

char finalArray[4];

int zeit = 500;
int led_zeit = 0;
byte contrast = 60;                          // Contrast für Display
byte bw = 0;                                 // Bandbreite 0 für 12.5KHz oder 1 für 25KHz
String bw_d = "S";
String bw_txt = "schmal";
double ftx;                                  // Tx Frequenz in MHz 134.0000 - 174.0000
double ftxk;
double frx;                                  // Rx Frequenz in MHz 134.0000 - 174.0000 
double frxk;
double volt;                                 // Spannung

String sh = "0";
byte ctcss = 0;
String ctcss_t = "0000";
String ctcssfre = "00.0";
int ctcss_test = 0;

byte squ = 1;                                // Squelch Pegel  ( 0 - 8 ); 0 = "offen"  
byte l = 2;                                  // Lautstärke 0-8
byte setup_power = 1;                        // 0-float , 0=500mW , float=1W muss invertiert sein
String power = "LOW";                        // da Transistor.
byte Pre_De = 0;                             // Pre/De ein
byte Hochpass = 0;                           // Hochpass ein
byte Tiefpass = 0;                           // Tiefpass ein
byte setupStatus = 0;    

const int setuppower = 7;                   // für Power Hi/Low Output 
const int setupPin = 10;                    // Pullup (a 4.7K resistor is necessary) 
const int downPin = 8;                      // Downtaster Pullup 
const int upPin = 9;                        // Uptaster Pullup 
const int led = 13;                         // TestLED 

byte addr = 0;                               // EEPROM Adresse 1
byte addrf = 6;

//#########################################################################################################

void setup()
{
  Serial.begin(9600);                        // open serial at 9600 bps,
  display.begin();
  display.setContrast(contrast);             //60 Kontrast ########################
  display.display();
  display.clearDisplay();                    // clears the screen and buffer
  pinMode(setupPin, INPUT);
  digitalWrite(setupPin, HIGH);              // turn on pullup resistors
  pinMode(setuppower, OUTPUT);
  digitalWrite(setuppower, LOW);  
  pinMode(downPin, INPUT);
  digitalWrite(downPin, HIGH);               // turn on pullup resistors
  pinMode(upPin, INPUT);
  digitalWrite(upPin, HIGH);                 // turn on pullup resistors
  pinMode(led, OUTPUT);
  digitalWrite(led, LOW);  

  delay(1000);
  
//############################# Speicher Initialisieren ###################################################

  if (digitalRead(setupPin) == LOW)                    // Initialisierung beim Start Frequenz,Leistung,Quelch,Raster,usw.
  {
    // AT+DMOSETGROUP=Bandbreite,Tx-Frequ.,Rx-Frequ.,TX-CTCSS,Squelch,Rx-CTCSS
    // AT+DMOSETGROUP=1,145.2750,145.2750,0000,2,0000

    frx = 1452750;                           // Rx Frequenz Startfrequenz
    ctcss = 0;                               // Tx/Rx CTCSS auf 0 setzen
    squ = 1;                                 // Squelchpegel
    l = 2;                                   // Lautstärke
    setup_power = 0;                         // Leistung 500mW
    bw = 1;
    frxk = frx / 1000;
    contrast = 60;
//    addr = 0;
//    addrf = 6;    
    digitalWrite(setuppower, LOW); 
    frequenz_uebergeben();

    EEPROM.write(addr,bw);                        // Babdbreite
    EEPROM.write(addr+1,squ);                     // Squelch
    EEPROM.write(addr+2,setup_power);             // Sendeleistung
    EEPROM.write(addr+3,ctcss);                   // CTCSS Frequenz
    EEPROM.write(addr+4,l);                       // Lautstärke
    EEPROM.write(addr+5,contrast);                // Contrast für Display
    
    char* byteArray = reinterpret_cast<char*>(&frx);
    for(int i=0;i<4;i++){EEPROM.write(addrf+i,byteArray[i]);}

    display.clearDisplay();                   // clears the screen and buffer
    display.setTextSize(1);
    display.setTextColor(BLACK);
    display.setCursor(28,0);                  // Courser Horizontal, Vertikal in Pixel
    display.print("Init");
    display.setCursor(22,10);                 // Courser Horizontal, Vertikal in Pixel
    display.print("fertig");
    display.setTextSize(1);
    display.setCursor(27,20);
    display.print("Taste");    
    display.setCursor(17,30);
    display.print("loslassen");       
    display.setCursor(17,40);
    display.print("(c) DL3ED");
    
    display.display();
    delay(2000);    
  }  
  else {
    bw = EEPROM.read(addr);                   // Bandbreite
    squ = EEPROM.read(addr+1);                // Squelch
    setup_power = EEPROM.read(addr+2);        // Sendeleistung
    ctcss = EEPROM.read(addr+3);              //CTCSS
    l = EEPROM.read(addr+4);                  // Lautstärke
    contrast = EEPROM.read(addr+5);
    
    for(int i=0;i<4;i++){delay(20);finalArray[i]=EEPROM.read(addrf+i);}
    frx = *reinterpret_cast<double*>(finalArray);
  }
    ctcss_frequenz_ermitteln();
    power_ermitteln(); 
    bw_ermitteln();   
    frequenz_waehlen();
    frequenz_uebergeben();
    frequenz_display();  
}

//#######################################################################################################

void loop()
{
led_zeit = led_zeit ++;                          // Zum Testen ob die SW läuft !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
if (led_zeit == 15000){
  led_zeit = 0;
  if (digitalRead(led) == LOW){digitalWrite(led,HIGH);}    
    else {digitalWrite(led,LOW);
  }
}
/*  
    if (ctcss_test < 9999){                       // Test für CTCSS einstellung

    ctcss_test = ctcss_test ++ ;
    ctcss_t = String(ctcss_test,DEC);    

    if (ctcss_test <= 9){Serial.println("000"+ctcss_t);ctcss_t=("000"+ctcss_t);}
    if (ctcss_test >= 10 && ctcss_test <= 99){Serial.println("00"+ctcss_t);ctcss_t=("00"+ctcss_t);}
    if (ctcss_test >= 100 && ctcss_test <= 999){Serial.println("0"+ctcss_t);ctcss_t=("0"+ctcss_t);}    
    
    frequenz_uebergeben();
    display.clearDisplay();                        // clears the screen and buffer
    display.setTextSize(2);
    display.setTextColor(BLACK);
    display.setCursor(0,0);
    display.print(ctcss_t);
    display.display();    
    }                                               // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  }
}
*/

if (digitalRead(downPin)==HIGH && digitalRead(upPin)==HIGH){zeit=500;}
  
  if (digitalRead(downPin) == LOW){               // Frequenzeinstellung 

    if (frx <= 1740000 && frx > 1460000){frx = frx - 100;goto down_goto;}
      else {if (frx <= 1460000 && frx > 1440000){frx = frx - 125;}
              else {if (frx <= 1440000 && frx >= 1340000){frx = frx - 100;}}
    }
    down_goto:         

    ctcss_frequenz_ermitteln();
    power_ermitteln();    
    frequenz_waehlen();
    frequenz_uebergeben();
    frequenz_display();
    if (zeit > 50){zeit = zeit - 50;}
    delay(zeit);
  }  
  if (digitalRead(upPin) == LOW){     

    if (frx >= 1340000 && frx < 1440000){frx = frx + 100;goto up_goto;}
      else {if (frx >= 1440000 && frx < 1460000){frx = frx + 125;}
              else {if (frx >= 1460000 && frx <= 1740000){frx = frx + 100;}}
    }
    up_goto:         

    ctcss_frequenz_ermitteln();
    power_ermitteln();    
    frequenz_waehlen();
    frequenz_uebergeben();
    frequenz_display();
    if (zeit > 50){zeit = zeit - 50;}    
    delay(zeit);    
  }  
  
  if (digitalRead(setupPin) == LOW){      
    setup_transceiver(); 
    frequenz_uebergeben();
    frequenz_display();
  }
}

//#######################################################################################################

//*************************** Unterprogramme ************************************************************

void setup_transceiver(){
  setup_lautstaerke();
  setup_display_ctcss();  
  setup_squelch();  
  setup_power_pegel();
  setup_bandbreite();
  setup_contrast();
  betriebsspannung();
  delay(200);  
}

//########################################################################################################

void setup_contrast(){

do{                                              // loop Lautstärke
  if (digitalRead(downPin) == LOW){     
    if (contrast == 20); 
    else {contrast = contrast - 5 ;
      display.setContrast(contrast);}
  }  
  if (digitalRead(upPin) == LOW) {     
    if (contrast == 120);
    else {contrast = contrast + 5 ;
      display.setContrast(contrast);}  
  }
  
    display.clearDisplay();                       // clears the screen and buffer
    display.setTextSize(2);
    display.setTextColor(BLACK);
    display.setCursor(0,0);                       // Courser Horizontal, Vertikal in Pixel
    display.print("Contr.");
    display.setCursor(0,24);
    display.print(contrast);    
    display.display();
    delay (300);
    }
    while   (digitalRead(setupPin) == HIGH);   
    EEPROM.write(addr+5,contrast);                       // Lautstärke speichern
    delay(200);
}

//#########################################################################################################

void betriebsspannung(){

  do{ 
    volt = analogRead(0);                         // AN0, Spannung lesen
    volt = (volt * 682) / 102400;                        
   
    display.clearDisplay();                     // clears the screen and buffer  
    display.setTextSize(1);
    display.setTextColor(BLACK);
    display.setTextSize(1);
    display.setCursor(20,0);
    display.print("Betriebs-");    
    display.setCursor(20,10);
    display.print("spannung");        
    display.setCursor(33,40);
    display.print("Volt");
    display.setTextSize(2);    
    display.setCursor(15,20);
    display.print(volt,3);   
    display.display();
    delay(200);
   }
   while   (digitalRead(setupPin) == HIGH); 
   delay(200);
}

//########################################################################################################

void setup_bandbreite(){

  do{ 
  if (digitalRead(downPin) == LOW){                    // Tastenstatus
    if (bw == 1);bw = 0;
      bw_ermitteln();
      frequenz_uebergeben();}  

  if (digitalRead(upPin) == LOW){                       // Tastenstatus
    if (bw == 0);bw =1 ;
      bw_ermitteln();
      frequenz_uebergeben();}

    display.clearDisplay();                     // clears the screen and buffer
    display.setTextSize(2);
    display.setTextColor(BLACK);
    display.setCursor(0,0);                     // Courser Horizontal, Vertikal in Pixel
    display.print("Bandbr.");
    display.setCursor(0,24);
    display.print(bw_txt);
    display.display();
    delay(200);
   }
   while   (digitalRead(setupPin) == HIGH); 
   EEPROM.write(addr,bw);                       // Bandbreite speichern
   delay(200);
}

void bw_ermitteln(){
   if (bw == 0){bw_txt = "schmal";bw_d = "S";}  
   else {bw = 1;bw_txt = "breit";bw_d = "B";}  
}

//########################################################################################################

void setup_power_pegel(){

  do{ 
  if (digitalRead(downPin) == LOW){                    // Tastenstatus
    if (setup_power == 1);
      setup_power = 0;
      power_ermitteln();
      frequenz_uebergeben();}  

  if (digitalRead(upPin) == LOW){                       // Tastenstatus
    if (setup_power == 0);
      setup_power = 1;
      power_ermitteln();
      frequenz_uebergeben();}

    display.clearDisplay();                     // clears the screen and buffer
    display.setTextSize(2);
    display.setTextColor(BLACK);
    display.setCursor(0,0);                     // Courser Horizontal, Vertikal in Pixel
    display.print("TxPower");
    display.setCursor(0,24);
    display.print(power);
    display.display();
    delay(200);
   }
   while   (digitalRead(setupPin) == HIGH); 
   EEPROM.write(addr+2,setup_power);             // Sendeleistung speichern
}

void power_ermitteln(){
   if (setup_power == 0){digitalWrite(setuppower, HIGH); power = "Low";}  
   else {digitalWrite(setuppower, LOW); power = "Hi";}  
}

//########################################################################################################

void setup_display_ctcss(){

  do{                                             //loop loop_tx_ctcss
  if (digitalRead(downPin) == LOW){    
    if (ctcss == 0); 
    else {ctcss = ctcss -- ;
      ctcss_frequenz_ermitteln();
      frequenz_uebergeben();}
  }  
  if (digitalRead(upPin) == LOW) {   
    if (ctcss == 38);
    else {ctcss = ctcss ++ ;
      ctcss_frequenz_ermitteln();
      frequenz_uebergeben();}  
   }

    display.clearDisplay();                        // clears the screen and buffer
    display.setTextSize(2);
    display.setTextColor(BLACK);
    display.setCursor(0,0);                        // Courser Horizontal, Vertikal in Pixel
    display.print("CTCSS");
    display.setCursor(0,24);
    display.print(ctcssfre);
    display.print("Hz");      
    display.display();
    delay (300);
  } 
  while   (digitalRead(setupPin) == HIGH); 
  EEPROM.write(addr+3,ctcss);                      // CTCSS Frequenz speichern
  delay(200);
}

void ctcss_frequenz_ermitteln()
{
        if (ctcss == 0){ ctcssfre = "00,0";}  
   else if (ctcss == 1){ ctcssfre = "67,0";}   
   else if (ctcss == 2){ ctcssfre = "71,9";}       
   else if (ctcss == 3){ ctcssfre = "74,4";}       
   else if (ctcss == 4){ ctcssfre = "77,0";}       
   else if (ctcss == 5){ ctcssfre = "79,7";}   
   else if (ctcss == 6){ ctcssfre = "82,5";}       
   else if (ctcss == 7){ ctcssfre = "85,4";}       
   else if (ctcss == 8){ ctcssfre = "88,5";}         
   else if (ctcss == 9){ ctcssfre = "91,5";}   
   else if (ctcss == 10){ ctcssfre = "94,8";}       
   else if (ctcss == 11){ ctcssfre = "97,4";}       
   else if (ctcss == 12){ ctcssfre = "100,0";}         
   else if (ctcss == 13){ ctcssfre = "103,5";}   
   else if (ctcss == 14){ ctcssfre = "107,2";}       
   else if (ctcss == 15){ ctcssfre = "110,9";}       
   else if (ctcss == 16){ ctcssfre = "114,8";}         
   else if (ctcss == 17){ ctcssfre = "118,8";}   
   else if (ctcss == 18){ ctcssfre = "123,0";}       
   else if (ctcss == 19){ ctcssfre = "127,3";}       
   else if (ctcss == 20){ ctcssfre = "131,8";}         
   else if (ctcss == 21){ ctcssfre = "136,5";}   
   else if (ctcss == 22){ ctcssfre = "141,3";}       
   else if (ctcss == 23){ ctcssfre = "146,2";}       
   else if (ctcss == 24){ ctcssfre = "151,4";}       
   else if (ctcss == 25){ ctcssfre = "156,7";}   
   else if (ctcss == 26){ ctcssfre = "162,2";}       
   else if (ctcss == 27){ ctcssfre = "167,9";}       
   else if (ctcss == 28){ ctcssfre = "173,8";}         
   else if (ctcss == 29){ ctcssfre = "179,9";}   
   else if (ctcss == 30){ ctcssfre = "186,2";}       
   else if (ctcss == 31){ ctcssfre = "192,8";}       
   else if (ctcss == 32){ ctcssfre = "203,5";}         
   else if (ctcss == 33){ ctcssfre = "210,7";}   
   else if (ctcss == 34){ ctcssfre = "218,1";}       
   else if (ctcss == 35){ ctcssfre = "225,7";}       
   else if (ctcss == 36){ ctcssfre = "233,6";}         
   else if (ctcss == 37){ ctcssfre = "241,8";}   
   else if (ctcss == 38){ ctcssfre = "250,3";}       

   ctcss_t = String(ctcss,DEC);
   if (ctcss <= 9){ctcss_t = ("000" + ctcss_t);}
     else {ctcss_t = ("00" + ctcss_t);}
}

//########################################################################################################

void setup_lautstaerke(){

do{                                              // loop Lautstärke
  if (digitalRead(downPin) == LOW){     
    if (l == 0); 
    else {l = l -- ;
      Serial.print("AT+DMOSETVOLUME="); 
      Serial.println(l);}
  }  
  if (digitalRead(upPin) == LOW) {     
    if (l == 8);
    else {l = l ++ ;
      Serial.print("AT+DMOSETVOLUME="); 
      Serial.println(l);}  
  }
  
    display.clearDisplay();                       // clears the screen and buffer
    display.setTextSize(2);
    display.setTextColor(BLACK);
    display.setCursor(0,0);                       // Courser Horizontal, Vertikal in Pixel
    display.print("Lautst.");
    display.setCursor(0,24);
    display.print(l);
    display.display();
    delay (300);
    }
    while   (digitalRead(setupPin) == HIGH);   
    EEPROM.write(addr+4,l);                       // Lautstärke speichern
    delay(200);
}

//########################################################################################################

void setup_squelch(){

  do{                                            //loop Squelch
  if (digitalRead(downPin) == LOW){    
    if (squ == 0); 
    else {squ = squ -- ;
      frequenz_uebergeben(); }
  }  
  if (digitalRead(upPin) == LOW){     
    if (squ == 8);
    else {squ = squ ++ ;
      frequenz_uebergeben();}  
  }
    display.clearDisplay();                     // clears the screen and buffer
    display.setTextSize(2);
    display.setTextColor(BLACK);
    display.setCursor(0,0);                     // Courser Horizontal, Vertikal in Pixel
    display.print("Squelch");
    display.setCursor(0,24);
    display.print(squ);
    display.display();
    delay (300);
    }
    while   (digitalRead(setupPin) == HIGH);   
    EEPROM.write(addr+1,squ);                     // Squelch speichern
    delay(200);
}

//########################################################################################################

void frequenz_waehlen()
{
  ftx = frx;
  sh = "0";
  if (frx >= 1456000 && frx <= 1457500){sh = "-";ftx = ftx - 6000;}
  frxk = frx / 10000;
  ftxk = ftx / 10000;
                                        
  char* byteArray = reinterpret_cast<char*>(&frx);          // Frequenzen wieder speichern
  for(int i=0;i<4;i++){EEPROM.write(addrf+i,byteArray[i]);}  
}  

//###########################################################################################################

void frequenz_uebergeben()
{
  // Protokoll
  // AT+DMOSETGROUP=Bandbreite,Tx-Frequ.,Rx-Frequ.,TX-CTCSS,Squelch,Rx-CTCSS
  // AT+DMOSETGROUP=0,145.2750,145.2750,0000,1,0000

    Serial.print("AT+DMOSETGROUP=");           // begin message but don't send yet..
    Serial.print(bw);                          // Bandbreite
    Serial.print(",");
    Serial.print(ftxk,4);                       // Tx Frequenz
    Serial.print(",");
    Serial.print(frxk,4);                       // Rx Frequenz
    Serial.print(",");
    Serial.print(ctcss_t);                     // Tx CTCSS
    Serial.print(",");
    Serial.print(squ);                         // Squelchpegel
    Serial.print(",");
    Serial.println(ctcss_t);                   // Rx CTCSS
    delay(10);    

    //AT+DMOSETVOLUME=x <CR><LF>
    Serial.print("AT+DMOSETVOLUME=");          // Lautstärke
    Serial.println(l);                          
    delay(10);

    //AT+ SETFILTER=PRE/DE-EMPH,Highpass,Lowpass <CR><LF>
    Serial.print("AT+SETFILTER=");             // Pre/De,Hochpass,Tiefpass
    Serial.print(Pre_De);
    Serial.print(",");    
    Serial.print(Hochpass);
    Serial.print(",");    
    Serial.println(Tiefpass);    
    
}

//##############################################################################################################

void frequenz_display()
{
    display.clearDisplay();                     // clears the screen and buffer
    display.setTextSize(2);                     // Schriftgröße
    display.setTextColor(BLACK);

    display.setCursor(0,0);                     // Courser Horizontal, Vertikal in Pixel
    display.print(frxk,3);
    display.setTextSize(1);

    display.setCursor(65,20);
    display.print("B.");    
    
    display.setCursor(77,20);
    display.print(bw_d);        

    display.setCursor(0,20);
    display.print("Power");
    
    display.setCursor(36,20);
    display.print(power);    
    
    display.setCursor(0,30);
    display.print("Sh.   Sq.  L.");
    
    display.setCursor(18,30);
    display.print(sh);

    display.setCursor(53,30);    
    display.print(squ);

    display.setCursor(77,30);
    display.print(l);
    
    display.setCursor(0,40);
    display.print("CTCSS       Hz");
    
    display.setCursor(36,40);
    display.print(ctcssfre);
    
    display.display();
}

